#!/bin/sh
#
# Component: taskd
#
# Description:
#    Runs shell tasks.
#
# Usage:
#    runshellcmd.sh [ -C | -P | -R | -E | -D | -O | -L | -I | -A | -G <property> ] { -f <task file> | <task cmd> <task arguments> ...}
#
#  -C               : cleanup task
#  -P               : print task
#  -R               : run task (default)
#  -E				: get resources of task - NOT USED, -A is used instead 
#  -D				: get display name of task - NOT USED, -A is used instead
#  -O				: batch open
#  -L				: batch close
#  -I				: get duplicate ID - NOT USED, -A is used instead
#  -A 			    : get All information (duplicateID, resources, display name, properties), separated
#  -G <property> 	: getting property value - NOT USED, -A is used instead
#					   Properties possible: 1PST - Study, 1PPE - Peer
#  -f <task file>   : full path of the task to be executed
#                     (cmd and arguments will be read from the task file)
#  <task cmd>       : the task command to be executed
#  <task arguments> : arguments to be passed to task command
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#

. $HOME/etc/setenv.sh

###############################
# Variables holding Arguments #
###############################

Mode=""
File=""
Silent=""
TASKD_JOBID=""
TASKD_SUBJOBID=""
PROPERTY=""

##################################
# runshellcmd specific functions #
##################################

# Prints usage with optional error message and exit
#
# [$1]  - error message
function usage_()
{
	if [ -n "$Silent" ] ; then
		[ -n "$1" ] && exit 1
		exit 0
	fi

	[ -n "$1" ] && echo "$1" >&2
	cat <<-ENDOFUSAGE
		$0 [-h] [ -C | -P | -R | -E | -D | -O | -L | -I | -A ] [-r retry] [-j jobid] [-u subjobid] [-s] [ -G property ] { -f <task file> | <task cmd> <task arguments> ...}

			-h               : prints this help
			-C               : cleanup task
			-P               : print task
			-R               : run task (default)
			-E               : get resources of task - NOT USED, -A is used instead
			-D			   	 : get display name of task - NOT USED, -A is used instead
			-O			     : batch open 
			-L			     : batch close
			-I			     : get duplicate ID - NOT USED, -A is used instead
			-G <property>	 : getting property value - NOT USED, -A is used instead
			-A 			     : get All information (duplicateID, resources, display name, properties), separated
			-r               : number of current (re)try
			-j			     : the job id of the task
			-u			     : the subjob id of the task
			-s               : silent mode (do not print even error messages)
			-f <task file>   : full path of the task to be executed
		                     (cmd and arguments will be read from the task file)
			<task cmd>       : the task command to be executed
			<task arguments> : arguments to be passed to task command

	ENDOFUSAGE

	[ -n "$1" ] && exit 1
	exit 0
}

# Prints error message on standard error and exits with exit code
#
# $1 exit code
# $2 error message
#
function errorExit_()
{
	[ -z "$Silent" ] && echo "$2" >&2
	exit $1
}

#######################################
# Default methods and common settings #
#######################################

DEFAULT_MAX_RETRY_NUM=-1

[ -f "$TASKD_CONF" ] && . $TASKD_CONF

#
# Default run method
#
run_()
{
	[ -z "$Silent" ] && echo "default run"
	return 0
}

#
# Default calculate retry min method
#
calculate_retry_min_()
{
	[ -z "$RETRY_NUM" ] && RETRY_NUM=0

	[ $DEFAULT_MAX_RETRY_NUM -gt 0 -a $RETRY_NUM -gt $DEFAULT_MAX_RETRY_NUM ] && return 125

	[ $RETRY_NUM -lt 5  ] && return 2    #  2 min
	[ $RETRY_NUM -lt 10 ] && return 10   # 10 min
	[ $RETRY_NUM -lt 14 ] && return 30   # 30 min
	[ $RETRY_NUM -lt 17 ] && return 60   # 60 min
	[ $RETRY_NUM -lt 20 ] && return 92   #  2 hour
	[ $RETRY_NUM -lt 22 ] && return 96   #  6 hour
	[ $RETRY_NUM -lt 26 ] && return 102  # 12 hour
	[ $RETRY_NUM -lt 30 ] && return 111  #  1 day
	[ $RETRY_NUM -lt 40 ] && return 112  #  2 day

	return 115
}

#
# init watson message
#
init_watson_message_()
{
	WATSON_MESSAGE_TEXT=""
	WATSON_MESSAGE_ID=""
	WATSON_MESSAGE_TYPE=ERROR
	WATSON_MESSAGE_LEVEL=TMP
}

#
# retrieve watson message
#
get_watson_message_()
{
	init_watson_message_
}

#
# call drwatson
#
call_dr_watson_()
{
	init_watson_message_

	get_watson_message_

	if [ -n "$WATSON_MESSAGE_TEXT" -a -n "$WATSON_MESSAGE_ID" ] ; then
		[ -z "$WATSON_MESSAGE_TYPE" ] && WATSON_MESSAGE_TYPE=ERROR
		[ -z "$WATSON_MESSAGE_LEVEL" ] && WATSON_MESSAGE_LEVEL=TMP

		$DRWATSON_HOME/addmessage.sh -i "$WATSON_MESSAGE_ID" -t "$WATSON_MESSAGE_TYPE" -l "$WATSON_MESSAGE_LEVEL" "$WATSON_MESSAGE_TEXT"
	fi
}

#
# cleanup task
#
cleanup_()
{
	return
}

#
# Print task
#
print_()
{
	echo "Unknown Task"
}


#
# default get resources 
#
get_resources_()
{
	echo "System"
}


#
# default get display name (for subjob)
#
get_display_name_()
{
	echo "Unknown Shell Script"
}


#
# default batch open
#
batch_open_()
{
	return 0 
}


#
# default batch close
#
batch_close_()
{
	return 0 
}


#
# default get duplicate ID
#
get_duplicate_id_()
{
	return 0
}


#
# default get property
#
get_def_property_()
{
	if [ "$PROPERTY" = "1PST" ] ; then
		echo "Unknown study" 
	else 
		if [ "$PROPERTY" = "1PPE" ] ; then
			echo "Unknown peer" 
		else
			echo "Unknown Property" 
		fi
	fi
}

get_property_() {
	get_def_property_
}

######################################
# Process arguments and prepare task #
######################################
while getopts 'hsCPREDOLIAG:r:f:j:u:' OPT; do
	case $OPT in
		h) usage_ ;;
		s) Silent="true" ;;
		C) if [ -n "$Mode" ] ; then usage_ "'-C', '-P', 'E', 'D', 'O', 'L', 'G', 'I', 'A' and '-R' options are mutually exclusive" ; else Mode="cleanup" ; fi ;;
		P) if [ -n "$Mode" ] ; then usage_ "'-C', '-P', 'E', 'D', 'O', 'L', 'G', 'I', 'A' and '-R' options are mutually exclusive" ; else Mode="print" ; fi ;;
		R) if [ -n "$Mode" ] ; then usage_ "'-C', '-P', 'E', 'D', 'O', 'L', 'G', 'I', 'A' and '-R options are mutually exclusive" ; else Mode="run" ; fi ;;
		E) if [ -n "$Mode" ] ; then usage_ "'-C', '-P', 'E', 'D', 'O', 'L', 'G', 'I', 'A' and '-R' options are mutually exclusive" ; else Mode="getresources" ; fi ;;
		D) if [ -n "$Mode" ] ; then usage_ "'-C', '-P', 'E', 'D', 'O', 'L', 'G', 'I', 'A' and '-R' options are mutually exclusive" ; else Mode="getdisplayname" ; fi ;;
		O) if [ -n "$Mode" ] ; then usage_ "'-C', '-P', 'E', 'D', 'O', 'L', 'G', 'I', 'A' and '-R' options are mutually exclusive" ; else Mode="batchopen" ; fi ;;
		L) if [ -n "$Mode" ] ; then usage_ "'-C', '-P', 'E', 'D', 'O', 'L', 'G', 'I', 'A' and '-R' options are mutually exclusive" ; else Mode="batchclose" ; fi ;;
		I) if [ -n "$Mode" ] ; then usage_ "'-C', '-P', 'E', 'D', 'O', 'L', 'G', 'I', 'A' and '-R' options are mutually exclusive" ; else Mode="getduplicateid" ; fi ;;
		A) if [ -n "$Mode" ] ; then usage_ "'-C', '-P', 'E', 'D', 'O', 'L', 'G', 'I', 'A' and '-R' options are mutually exclusive" ; else Mode="getallinfo" ; fi ;;
		G) if [ -n "$Mode" ] ; then usage_ "'-C', '-P', 'E', 'D', 'O', 'L', 'G', 'I', 'A' and '-R' options are mutually exclusive" ; 
						else 
				Mode="getproperty" 
				PROPERTY="$OPTARG" ; 
			fi ;;
		f) File="$OPTARG" ;;
		r) RETRY_NUM="$OPTARG" ;;
		j) TASKD_JOBID="$OPTARG" ;;
		u) TASKD_SUBJOBID="$OPTARG" ;;
		*) usage_ "Invalid Parameter: $OPT" ;;
	esac
done

CMD=""

if [ -n "$File" ] ; then
	[ ! -f "$File" ] && errorExit_ -1 "File $File does not exist"

	STR="`grep -v '^#' $File`"
	CMD=`cut -f 1 -d " " <<<$STR`
	ARG1="${STR#$CMD }"

	[ -z "$CMD" ] && errorExit_ -1 "File $File does not contain command"

	eval "set -- $ARG1"
else
	shift $(($OPTIND-1))
	CMD="$1"

	[ -z "$CMD" ] && usage_ "One of the -f <task file> or <task cmd> option/argument should be specified"
	[ ! -f "$CMD" ] && errorExit_ -1 "Command $CMD does not exist"

	shift
fi

###########################################################
# Include task command file to override default functions #
###########################################################

# Re-initialize OPTIND
OPTIND=1

. $CMD

[ $? -ne 0 ] && errorExit_ -1 "Could not include $CMD"

[ -z "$RETRY_NUM" ] && RETRY_NUM=0

#
# Check and execute cleanup
#
if [ "$Mode" = "cleanup" ] ; then
	cleanup_
	exit 0
fi

#
# Check and execute print
#
if [ "$Mode" = "print" ] ; then
	print_
	exit 0
fi

#
# Check and execute getresources
#
if [ "$Mode" = "getresources" ] ; then
	get_resources_
	exit 0
fi

#
# Check and execute getdisplayname
#
if [ "$Mode" = "getdisplayname" ] ; then
	get_display_name_
	exit 0
fi

#
# Check and execute batchopen
#
if [ "$Mode" = "batchopen" ] ; then
	batch_open_
	exit $?
fi

#
# Check and execute batchclose
#
if [ "$Mode" = "batchclose" ] ; then
	batch_close_
	exit $?
fi

#
# Check and execute getduplicateid
#
if [ "$Mode" = "getduplicateid" ] ; then
	get_duplicate_id_
	exit 0
fi

#
# Check and execute getproperty
#
if [ "$Mode" = "getproperty" ] ; then
	get_property_ 
	exit 0
fi

#
# Check and execute getallinfo
#
if [ "$Mode" = "getallinfo" ] ; then
	echo -n "resources###"
	get_resources_
	
	echo -n "###duplicateid###"
	get_duplicate_id_

	echo -n "###displayname###"
	get_display_name_

	echo -n "###properties###"
	PROPERTY="1PST"					
	echo -n $PROPERTY"="
	get_property_

	PROPERTY="1PPE"					
	echo -n $PROPERTY"="
	get_property_
																																																																																																																																																																																																																																																												
	exit 0
fi

#
# Run task
#

# run_ return values are expected as:
# 0          : Success
# 1 - 124    : Temporal Error - needs to retry
# 125 =<     : Fatal Error    - failed
#
# in old versions:
# < 0        : Fatal Error   - failed
#
# but newer shell versions convert signed return values to unsigned
RETRY_MIN=0

export TASKD_JOBID TASKD_SUBJOBID

if [ -n "$Silent" ] ; then
	run_ >/dev/null 2>&1
	RUN_RESULT=$?
else
	run_
	RUN_RESULT=$?
fi

[ $RUN_RESULT -lt 0 ] && RUN_RESULT=125

if [ $RUN_RESULT -ne 0 ] ; then

	if [ $RUN_RESULT -gt 0 -a $RUN_RESULT -le 124 ] ;then
		calculate_retry_min_

		RETRY_MIN=$?
	else
		RETRY_MIN=125
	fi

else
	if [ -n "$Silent" ] ; then
		cleanup_ >/dev/null 2>&1
	else
		cleanup_
	fi
fi

if [ -n "$Silent" ] ; then
	call_dr_watson_ >/dev/null 2>&1
else
	call_dr_watson_
fi

exit $RETRY_MIN
